package Real::Handy::NoConfig;
${'Real::Handy::SKIP_CONFIG'} = 1 for 1;
require Real::Handy;
Real::Handy::set_autouse( "Data::Dumper=Dumper" );
Real::Handy::set_autouse( "Scalar::Util=weaken" );
Real::Handy::set_autouse( "Digest::MD5=md5_hex,md5_base64" );
Real::Handy::set_autouse( "Carp=croak,carp,confess" );
Real::Handy::set_autouse( "List::Util=min,max" );
Real::Handy::set_autouse( "DBI" );
Real::Handy::set_autouse( "FCGI" );
Real::Handy::set_autouse( "POSIX=strftime" );
@ISA = 'Real::Handy';
